'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.FacadePragmatics = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

exports.createFacadeInitializer = createFacadeInitializer;
exports.createFacadePragmatics = createFacadePragmatics;

var _babelute = require('../babelute.js');

var _pragmaticsScopes = require('./pragmatics-scopes');

var _pragmaticsScopes2 = _interopRequireDefault(_pragmaticsScopes);

var _pragmaticsCore = require('./pragmatics-core.js');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } // removed in production
/**
 * @author Gilles Coomans
 * @licence MIT
 * @copyright 2016-2017 Gilles Coomans
 */

/**
 * FacadePragmatics : a facade oriented Pragmatics subclass. You should never instanciate a FacadePragmatics directly with new. use {@link createFacadePragmatics}.
 * @example
 * // Remarque : any lexem's method will be of the following format : 
 * function(subject, args, ?scopes){
 * 	// return nothing
 * }
 */
var FacadePragmatics = exports.FacadePragmatics = function (_Pragmatics) {
	_inherits(FacadePragmatics, _Pragmatics);

	/**
  * @param  {Object} targets initial targets object
  * @param  {?Object} pragmas pragmatics methods to add
  */
	function FacadePragmatics(targets) {
		var pragmas = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

		_classCallCheck(this, FacadePragmatics);

		return _possibleConstructorReturn(this, (FacadePragmatics.__proto__ || Object.getPrototypeOf(FacadePragmatics)).call(this, targets, pragmas));
	}

	/**
  * "each" facade implementation
  * @param  {Object} subject the handled subject
  * @param  {Array|arguments} args  the lexem's args : [ collection:Array, itemHandler:Function ]
  * @param  {Scopes} scopes  the sentence's scopes instance
  * @return {void}         nothing
  */


	_createClass(FacadePragmatics, [{
		key: 'each',
		value: function each(subject, args /* collection, itemHandler */, scopes) {
			var collec = args[0],
			    itemHandler = args[1];

			if (collec.length) // no supputation on collection kind : use "for"
				for (var i = 0, len = collec.length, item, templ; i < len; ++i) {
					item = collec[i];
					templ = itemHandler(item, i);
					if (templ) this.$output(subject, templ, scopes);
				}
		}

		/**
   * "if" facade implementation 
   * @param  {Object} subject the handled subject
   * @param  {Array|arguments} args  the lexem's args : [ conditionIsTrue:Babelute, conditionIsFalse:Babelute ]
   * @param  {Scopes} scopes  the sentence's scopes instance
   * @return {void}         nothing
   */

	}, {
		key: 'if',
		value: function _if(subject, args /* trueBabelute, falseBabelute */, scopes) {

			if (args[0]) this.$output(subject, args[1], scopes);else if (args[2]) this.$output(subject, args[2], scopes);
		}

		/**
   *
   * @override
   * @param  {Object} subject  the subject handle through interpretation
   * @param  {Babelute} babelute the babelute "to interpret on" subject
   * @param  {Scope} scopes   the sentence scopes instance (optional)
   * @return {Object}        the subject
   */

	}, {
		key: '$output',
		value: function $output(subject, babelute) {
			var scopes = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;


			for (var i = 0, lexem, len = babelute._lexems.length; i < len; ++i) {
				lexem = babelute._lexems[i];
				if (this._targets[lexem.lexicon] && this[lexem.name]) this[lexem.name](subject, lexem.args, scopes);
			}
			return subject;
		}
	}]);

	return FacadePragmatics;
}(_pragmaticsCore.Pragmatics);

/**
 * create a facade-ready-to-run initializer function.
 * @param  {Lexicon} lexicon    the lexicon from where take the api
 * @param  {Object} pragmatics   the pragmatics object where to find interpretation method to fire immediatly
 * @return {Function}            the facade initializer function
 * @example
 *
 * import babelute from 'babelute';
 * const myLexicon = babelute.createLexicon('my-lexicon');
 * myLexicon.addAtoms(['foo', 'bar']);
 * 
 * const myPragmas = babelute.createFacadePragmatics({
 * 	'my-lexicon':true
 * }, {
 * 	foo(subject, args, scopes){
 * 		// do something
 * 	},
 * 	bar(subject, args, scopes){
 * 		// do something
 * 	}
 * });
 *
 * const mlp = babelute.createFacadeInitializer(myLexicon, myPragmas);
 *
 * mlp(mySubject).foo(...).bar(...); // apply pragmas immediatly on subject through lexicon api's
 *
 */


function createFacadeInitializer(lexicon, pragmatics) {
	var Facade = function Facade(subject, scopes) {
		lexicon.Atomic.call(this);
		this._subject = subject;
		this._scopes = scopes;
	};

	Facade.prototype = Object.create(lexicon.Atomic.prototype);
	Facade.prototype.constructor = Facade;
	Facade.prototype._lexicon = null;
	Facade.prototype._append = function (lexiconName, name, args) {
		if ((!pragmatics._targets || pragmatics._targets[lexiconName]) && pragmatics[name]) pragmatics[name](this._subject, args, this._scopes);
		return this;
	};
	return function (subject) {
		var scopes = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

		return new Facade(subject, scopes || new _pragmaticsScopes2.default());
	};
}

/**
 * create a FacadePragmatics instance
 * @param  {Object} targets the pragmatics targets DSL
 * @param  {?Object} pragmas the methods to add
 * @return {FacadePragmatics}     the facade pragmatics instance
 * @example
 * const myPragmas = babelute.createFacadePragmatics({
 * 	'my-lexicon':true
 * }, {
 * 	foo(subject, args, scopes){
 * 		// do something
 * 	},
 * 	bar(subject, args, scopes){
 * 		// do something
 * 	}
 * });
 */
function createFacadePragmatics(targets) {
	var pragmas = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

	return new FacadePragmatics(targets, pragmas);
}